package tsp.csc4509.tpnote;

import static tsp.csc4509.common.Log.GEN;
import java.io.IOException;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.SocketChannel;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import tsp.csc4509.blockfile.Block;
import tsp.csc4509.blockfile.BlockFile;
import tsp.csc4509.tcp.TcpSocketSynchrone;

/**
 * Classe du client.
 * 
 * @author
 *
 */
public class Client {

	/**
	 * nombre de connexions simultanées ouvertes par le client.
	 */
	private static final int NBCONNECTIONS = 5;

	/**
	 * Fichier divisé en blocs attendus par le client.
	 */
	private BlockFile blockFile;

	/**
	 * map contenant l'ensemble des blocs actuellement en téléchargement (clef: le
	 * canal connecté au serveur).
	 */
	private Map<SocketChannel, Block> serverMap;

	/**
	 * selector utilisé par le client pour gérer son mode asynchrone.
	 */
	private Selector selector;

	/**
	 * Contructeur du client qui. (1) initialise file avec un File gérant le fichier
	 * fileName; (2) initialise serverMap avec une HashMap; (3) ouvre le selector.
	 * 
	 * @param fileName nom du fichier à importer.
	 * @throws IOException toutes les exceptions d'entrées sorties qui peuvent se
	 *                     produire.
	 */
	public Client(final String fileName) throws IOException {
		super();
		GEN.info("Construction du client pour " + fileName);
		blockFile = new BlockFile(fileName);
		serverMap = new HashMap<SocketChannel, Block>();
		selector = Selector.open();
	}

	/**
	 * Connecte le client au serveur et lui envoie une requête.
	 * 
	 * @param serverInfo coordonnées (hostname et port) du serveur.
	 * @param request    requête à envoyer.
	 * @return le canal de la connexion reliée au serveur.
	 * @throws IOException toutes les exceptions d'entrées/sorties qui peuvent se
	 *                     produire.
	 */
	public SocketChannel connectAndSendRequest(final ServerInfo serverInfo, final Request request) throws IOException {
		// Devrait être private.
		// Rendue public pour la classe de test.
		// TODO Question 1
		return null;
	}

	/**
	 * Méthode qui prend en charge un nouveau bloc du fichier à importer. Cette
	 * méthode doit: (1) obtenir un bloc du fichier qui n'est pas encore téléchargé
	 * (s'il en reste); (2) créer un nouvelle requête; (3) se connecter à un serveur
	 * pour envoyer cette requête; (4) passer en mode asynchrone, inscrire le canal
	 * relié au serveur dans le selector et ajouter ce bloc dans la map qui gère les
	 * blocs en téléchargement.
	 * 
	 * @throws IOException toutes les exceptions d'entrées/sorties qui peuvent se
	 *                     produire.
	 */
	public void manageNextBlock() throws IOException {
		// Devrait être private.
		// Rendue public pour la classe de test.
		// TODO Question 2
	}

	/**
	 * Méthode qui importe le fichier attendu par ce client. Cette méthode doit: (1)
	 * mettre en place la gestion des NBCONNECTIONS premiers blocs; et (2) tant
	 * qu'il reste un bloc à importer: (2a) gérer en simultané la réception des
	 * données de tous les blocs gérés; (2b) fermer tout bloc complété, et prendre
	 * en charge un nouveau bloc. Si le client perd la connexion avec un serveur
	 * avant que le bloc qu'il importe ne soit complet, il faut alors replacer ce
	 * blocs dans l'ensemble du fichier (pour qu'il puisse à nouveau être gérer plus
	 * tard), et prendre en charge un nouveau bloc.
	 * 
	 * @throws IOException toutes les exceptions d'entrées/sorties qui peuvent se
	 *                     produire.
	 */
	public void importFile() throws IOException {
		// TODO Question 3
	}

	// *****************
	// Getters sur les attributs ajoutés à la classe pour l'écriture des tests.
	// Ils ne doivent pas être utilisés dans votre code. Seule la classe Client doit
	// manipuler ces attributs.
	/**
	 * getter sur blockFile.
	 * 
	 * @return l'attribut blockFile.
	 */
	public BlockFile getBlockFile() {
		return blockFile;
	}

	/**
	 * getter sur serverMap.
	 * 
	 * @return l'attribut serverMap.
	 */
	public Map<SocketChannel, Block> getServerMap() {
		return serverMap;
	}

	/**
	 * getter sur selector.
	 * 
	 * @return l'attribut selector.
	 */
	public Selector getSelector() {
		return selector;
	}

}
