package tsp.csc4509.tpnote;

import java.util.Random;

/**
 * Annuaire des serveurs.
 * 
 * Ne pas modifier cette classe pour que les tests fonctionnent.
 * @author EricPro
 *
 */
public enum ServerInfo {
	/**
	 * Premier serveur. Peut s'obtenir en random avec un seed à 4096.
	 */
	SERVER1("localhost", (short) 4545),
	/**
	 * Second serveur. Peut s'obtenir en random avec un seed à 0.
	 */
	SERVER2("localhost", (short) 4546);
	

	
	/**
	 * nom de la machine hébergeant le serveur.
	 */
	 private String hostname;
	
	/**
	 * numéro du port utilisé par le serveur.
	 */
	private short port;
	
	/**
	 * tableau contenant toutes les valeurs de l'énumération.
	 */
	private static ServerInfo[] servers = ServerInfo.values();
	
	/**
	 * un générateur de valeurs pseudo-aléatoires.
	 */
	private static Random randGen = new Random();

	/**
	 * Constructeur initialisant hostname et port.
	 * @param hostname
	 *            nom de la machine hébergeant le serveur.
	 * @param port
	 *            numéro du port utilisé par le serveur.
	 */
	ServerInfo(final String hostname, final short port) {
		this.hostname = hostname;
		this.port = port;
	}

	/**
	 * getter sur le nom de la machine du serveur.
	 * @return
	 *        le nom de la machine du serveur.
	 */
	public String getHostname() {
		return hostname;
	}

	/**
	 * getteur sur le port du serveur.
	 * @return
	 *       le port du serveur.
	 */
	public short getPort() {
		return port;
	}
	
	/**
	 * getter sur un serveur pris au hasard dans les serveurs énumérés.
	 * @return
	 *        un de serveurs des énumérés.
	 */
	public static ServerInfo getRandomServer() {
		return servers[randGen.nextInt(servers.length)];
	}
	
	/**
	 * Fixe la graine du random pour choisir le hasard (pour aider les tests).
	 * Avec 2 serveurs dans l'énumération:
	 *     seed = 0 donne le SERVER2
	 *     seed = 4096 donne le SERVER1
	 * @param seed
	 *         valeur de la graine du random.
	 */
	public static void chooseRandom(final long seed) {
		randGen.setSeed(seed);
	}

}
